'SaveSettings.vbs - 1/11/2012 - MDR
'Utility script which will save customized registry settings, drive definitions, catalog database
'and pre- and post-.bat scripts so that they can be easily restored after a software update.
' 10/29/2013 - save settings now appends date stamp to foldername used to hold catalog. Restore will require date in foldername.
' 12/30/2014 - add ability to backup SQL database containing catalog (RBBackup)
' 12/23/2018 - add backupsetup.ini to saved files.

  CONST RBSProvider = "Dr.Backup"
  CONST Version = "(v1.2)"

  '** standard pre-amble for all interactive scripts **
  'get OS volume (do not assume it's C:\)
  Set OSobj = CreateObject("Scripting.FileSystemObject")
  OSfolder = OSobj.getspecialfolder(0)
  objStartFolder = Left(OSfolder, 3)  'root of search file tree
  OSVolume = Left(OSfolder, 1)

  '** first time through with modern OS, re-run script to elevate privs (first time args = 0)
  If WScript.Arguments.count = 0 AND NewOS() Then
    Set objShell = CreateObject("Shell.Application")
    'Pass a bogus argument with leading blank space, say [ uac]
    objShell.ShellExecute "wscript.exe", Chr(34) & _
    WScript.ScriptFullName & Chr(34) & " uac", "", "runas", 1
    Wscript.Quit
  End If

  'find location of 32-bit script processing program - in syswow64 on 64-bit machines
  ScriptEXE = OSFolder & "\SYSTEM32\Cscript.exe"
  Set objFSOexe = CreateObject("Scripting.FileSystemObject")
  If objFSOexe.FileExists(OSFolder & "\SYSWOW64\Cscript.exe") Then
    '64-bit system found. switch to 32-bit cscript engine for database drivers for MS Access
    ScriptEXE = OSFolder & "\SYSWOW64\Cscript.exe"
  End If

  '** if we have UAC escalated count will be 1, otherwise 0 on legacy machines **
  '** force use of cscript by re-running so we get console display **
  If Wscript.Arguments.Count < 2 Then
      strPath = Wscript.ScriptFullName
      strCommand = "%comspec% /k " & ScriptEXE & " //nologo """ & strPath & """" & " 1 2"
      Set objShell = CreateObject("Wscript.Shell")
      objShell.Run(strCommand), 1, True
      Wscript.Quit
  End If

  'On Error Resume Next
  'Err.Clear

  Wscript.StdOut.Write(RBSProvider & " - Save (Customized) Settings Utility " & Version & VbCrLF)
  Wscript.StdOut.WriteLine("Start: " & Now() & VbCrLf)

  'Prompt if ready to save customizations
  status = MsgBox("Stop all " & RBSProvider & " components. Ready?", vbOkCancel + vbInformation + vbDefaultButton2, "Save Settings")
  If status = vbCancel Then
    Wscript.Quit
  End If

  'Get program file location from Registry
  Const HKEY_LOCAL_MACHINE = &H80000002
  strComputer = "."
  strKeyPath = "SOFTWARE\Quantum Tech, Inc.\Remote Backup\Settings"
  strEntryName = "ClientDB"
  Set objReg = GetObject("winmgmts:" _
   & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\default:StdRegProv")
  status = objReg.GetStringValue( HKEY_LOCAL_MACHINE, strKeyPath, strEntryName, strValue)
  DBRegPath = strValue

  'Sometimes security products block access to the registry. If so, we'll quit here.
  If (status <> 0) or (Err.Number <> 0) Then 
    WScript.StdOut.Writeline ("Unable to access registry. Quitting." & VbCr)
    Wscript.Quit
  End If

  'Create a folder inside of the user's temporary folder to hold saved data
  'Get the environmental %temp% value as root for working folder
  Set wshShell = CreateObject("WScript.Shell" )
  TempPath = wshShell.ExpandEnvironmentStrings( "%TEMP%" )
  'Wscript.StdOut.WriteLine("Temp space located  : " & TempPath)
  CONST WorkFolder = "DrBackupSettings"       '%temp%\DrBackupSettings-month+day+year is destination folder
  dt = Now()
  strFolder = TempPath & "\" & WorkFolder & "-" & month(dt) & day(dt) & year(dt)
  'Create a working folder in temp area
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  If objFSO.FolderExists(strFolder) = FALSE then
    objFSO.CreateFolder strFolder
    Wscript.StdOut.WriteLine("Configuration Folder: " & strFolder)
  Else
    Wscript.StdOut.WriteLine("Configuration Folder (Reused): " & strFolder & VbCrLf)
  End If

  CONST OverwriteExisting = TRUE
  Err.Clear
  On Error Goto 0
  Wscript.StdOut.WriteLine("<Saving Files>")

  CONST NetUseFile = "rbnetuse.ini"
  'We now have the fully qualified path to the mapped drive definitions file (if it exists)
  MappedDrives = DBRegPath & "\" & NetUseFile
  'Wscript.StdOut.WriteLine("Mapped Drive Config : " & MappedDrives)
  'Copy the rbnetuse.ini file to settings Workfolder if it exists
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  If objFSO.FileExists(MappedDrives) = TRUE then
    objFSO.CopyFile MappedDrives, strFolder & "\", OverwriteExisting
    Wscript.StdOut.WriteLine("Copied Mapped drive settings.")
  Else
    Wscript.StdOut.WriteLine("Warn: No mapped drive settings.")
  End If

  CONST BackupSetupFile = "backupsetup.ini"
  'We now have the fully qualified path to the backupsetup config file (post V12)
  BackupSetup = DBRegPath & "\" & BackupSetupFile
  'Wscript.StdOut.WriteLine("BackupSetup Config : " & BackupSetup)
  'Copy the backupsetup.ini file to settings Workfolder if it exists
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  If objFSO.FileExists(BackupSetup) = TRUE then
    objFSO.CopyFile BackupSetup, strFolder & "\", OverwriteExisting
    Wscript.StdOut.WriteLine("BackupSetup configuration saved.")
  Else
    Wscript.StdOut.WriteLine("Warn: No backupsetup confguration (pre-V12)")
  End If

  'Copy any file of the format "pre-*" to the settings Workfolder
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  objFSO.CopyFile DBRegPath & "\" & "pre-*.*", strFolder & "\", OverwriteExisting
  Wscript.StdOut.WriteLine("Copied Pre-files.")

  'Copy any file of the format "post-*" to the settings Workfolder
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  objFSO.CopyFile DBRegPath & "\" & "post-*.*", strFolder & "\", OverwriteExisting
  Wscript.StdOut.WriteLine("Copied Post-files.")

  Wscript.StdOut.WriteLine(VbCrLf & "<Saving Registry Keys>")
  'Build a list of registry key strings we want to iterate through
  CONST RegistryKeys = "BufferSize|FreespaceThreshold|LocalTransfer|LocalTransferPath|MaxFileSizeLimit|MaxSyncTransactions|QueueLimit|StoreNameChars|CS|CSPath|DBName|DBPass|DBServer|DBType|DBUser"
  Dim RegKeys             'list of registry customizations
  Dim RegKeysSave         'will hold string of registry keys with data
  Dim RegKeysSaveValues   'corresponding values of RegKeysSave

  RegKeys = Split(RegistryKeys, "|")
  RegKeysSave = ""
  RegKeysSaveValues = ""
   For i=0 to Ubound(RegKeys)
   'Get catalog location from Registry
    strComputer = "."
    strKeyPath = "SOFTWARE\Quantum Tech, Inc.\Remote Backup\Settings"
    strEntryName = RegKeys(i)
    Set objReg = GetObject("winmgmts:" _
     & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\default:StdRegProv")
    status = objReg.GetStringValue( HKEY_LOCAL_MACHINE, strKeyPath, strEntryName, strValue)

    'Sometimes security products block access to the registry. If so, this script will exit.
    If (status <> 0) or (Err.Number <> 0) Then 
      'Cannot get key
      Wscript.StdOut.WriteLine("""" & RegKeys(i) & """" & " = " & "<Not Available>")          
    Else
      'fetched key. write it to output file for safekeeping
      RegKeysSave = RegKeysSave & "|" & RegKeys(i)
      RegKeysSaveValues = RegKeysSaveValues & "|" & strValue
      Wscript.StdOut.WriteLine("""" & RegKeys(i) & """" & "=" & """" & strValue & """"& " - saved.")
    End If
  Next

  'Wscript.StdOut.WriteLine("Keys   = " & Right(RegKeysSave, Len(RegKeysSave)-1))
  'Wscript.StdOut.WriteLine("Values = " & Right(RegKeysSaveValues, Len(RegKeysSaveValues)-1))

  CONST RegistryFile = "RegistryFile.txt"
  'save the registry data to a file.
  Const ForWriting = 2
  Set objFSO = CreateObject("Scripting.FileSystemObject")
  Set objFile = objFSO.CreateTextFile(strFolder & "\" & RegistryFile, OverwriteExisting)
  objFile.WriteLine Right(RegKeysSave, Len(RegKeysSave)-1)
  objFile.WriteLine Right(RegKeysSaveValues, Len(RegKeysSaveValues)-1)
  objFile.Close

  On Error Resume Next
  Err.Clear

  'backup the database here...MS Access or SQL (express)
  'INITIALIZE DATABASE TYPE
  'grab Database type field from registry
  strEntryName = "DBType"
  err.clear
  Set objReg = GetObject("winmgmts:" _
   & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\default:StdRegProv")
  status = objReg.GetStringValue( HKEY_LOCAL_MACHINE, strKeyPath, strEntryName, strValue)
  If (status = 0) and (Err.Number = 0) Then
    'dbtype field found.
    DBType = strValue
  Else
    DBType = "0"       'default to MS Access db
  End If

  If DBType <> "0" Then
    'If SQL Server, then we will need the connection string. Get it here.
    strEntryName = "DBServer"
    Err.Clear
    Set objReg = GetObject("winmgmts:" _
                 & "{impersonationLevel=impersonate}!\\" & strComputer & "\root\default:StdRegProv")
    status = objReg.GetStringValue( HKEY_LOCAL_MACHINE, strKeyPath, strEntryName, strValue)
    If (status = 0) and (Err.Number = 0) Then
      DBServer = strValue
      WScript.StdOut.Writeline (VbCrLf& "Processing SQL Server Database: " & DBServer & " (RBBACKUP)" & VbCrLf)
    Else
      WScript.StdOut.Writeline (VbCrLf & "SQL Registry setting error (DBServer). Quitting." & VbCrLf)
      WScript.Quit
    End If
  End If

  If DBType = "0" Then
    CONST BackupCatalog = "backup.mdb"
    'We now have the fully qualified path to the main catalog database
    Database = DBRegPath & "\" & BackupCatalog
    Wscript.StdOut.WriteLine(VbCrLf & "Catalog Location: " & Database)
    'Copy the catalog file to settings Workfolder
    Set objFSO = CreateObject("Scripting.FileSystemObject")
    objFSO.CopyFile Database, strFolder & "\", OverwriteExisting
    If Err.Number <> 0 Then 
      WScript.StdOut.Writeline (VbCrLf & "Unable to backup database. Quitting." & VbCrLF)
      Wscript.Quit
    Else
      Wscript.StdOut.WriteLine("Copied Database.")
    End If
  Else

    'prompt user to see if we want to do SQL db backup
    Response = InputBox("Skip backup of SQL Database ? Y/[N]: ","Default: Backup SQL Database", "")
    If UCase(Response) = "Y" Then
      'skip database backup 
      Wscript.StdOut.WriteLine("Skipping backup of SQL database: " & DBServer & "(RBackup)")
    Else
      'This is an SQL database catalog - back it up to \Remote Backup folder (due to permissions)
      'then, copy the .bak to the temp area with the other data
      CONST BackupSQLCatalog = "RBBackup.bak"
      Wscript.StdOut.WriteLine("Making SQL backup: " & DBServer & "(RBackup) as" & " " & DBRegPath & "\" & BackupSQLCatalog )
      BackupSQL DBServer, "RBBACKUP", DBRegPath & "\" & BackupSQLCatalog       '.bak overwrites itself in \Remote Backup folder.
      Database = DBRegPath & "\" & BackupSQLCatalog
      'Copy the catalog file to settings Workfolder
      Set objFSO = CreateObject("Scripting.FileSystemObject")
      objFSO.CopyFile Database, strFolder & "\", OverwriteExisting
      If Err.Number <> 0 Then 
        WScript.StdOut.Writeline ("Unable to copy SQL backup database. Quitting." & VbCrLF)
        Wscript.Quit
      Else
        Wscript.StdOut.WriteLine("Copied Backup of SQL Database.")
      End If
    End If
  End If

  'show complete message
  Wscript.StdOut.Writeline(" ")
  'Wscript.StdOut.WriteLine("Configuration data stored: " & strFolder)
  Wscript.StdOut.WriteLine("Complete: " & Now())
  Wscript.Quit

Function NewOS()
  'Utility function to determine if this is new machine, i.e., will have UAC if > 2003
  On Error Resume Next
  strComputer = "."
  OScaption = ""
  NewOS = False
  Set objWMIService = GetObject("winmgmts:\\" & strComputer & "\root\cimv2")        
  Set colOperatingSystems = objWMIService.ExecQuery _
      ("Select * from Win32_OperatingSystem")
  For Each objOperatingSystem in colOperatingSystems
   OScaption = objOperatingSystem.Caption
  Next
  If Instr(OScaption,"XP") > 0 OR _
     Instr(OScaption, "2003") > 0 OR _
     instr(OScaption, "2000") > 0 OR _
     len(OScaption) = 0 Then
    NewOS = False
  Else
    NewOS = True
  End If
  Exit Function
End Function

Sub BackupSQL(strServerName, strDatabaseName, strBackupFile)
'Backup SQL database to strDatabaseName.bak in folder strBackupfolder
'Example: BackupSQL ".\SQLEXPRESS", "RBBACKUP", "C:\SQLBackup\RBBACKUP.bak"

  Dim adoCommand
  Dim strConnectionstring
  Dim strfileName

  ' Specify backup path to save the .bak file.
  'strBackupPath = "C:\SQLBackup"

  'Specify the name of server and instance if present to which backup file need to be created .
  'strServerName =".\SQLEXPRESS "

  'Specify the database name to which backup file need to be generated.
  'strDatabaseName ="RBBACKUP"

  'Specify connection string to Master Database on SQL server as best practise.
  strConnectionstring= "Provider=SQLOLEDB.1;Integrated Security=SSPI;Persist Security Info=False;Initial Catalog=MASTER;Data Source=" & strServerName

  ' Backup the database .
  Set adoCommand = CreateObject("ADODB.Command")

  On Error Resume Next
  adoCommand.ActiveConnection = strConnectionstring
  If (Err.Number <> 0) Then
    Wscript.Echo "BackupSQL: Unable to connect to SQL database."
    Wscript.Echo Err.Description
    Wscript.Quit
  End If

  strfileName = strBackupFile
  adoCommand.CommandText = " Create database " & strDatabaseName
  adoCommand.CommandText = "Backup DATABASE " & strDatabaseName & " to disk='" & strBackupFile & "'" & " With INIT"
  adoCommand.CommandTimeout = 0
  adoCommand.Execute

  If (Err.Number <> 0) Then
    Wscript.Echo "BackupSQL: Unable to write SQL database."
    Wscript.Echo Err.Description
    Wscript.Quit
  End If

  'MsgBox "BackUp Completed Successfully"
  Set adoCommand = Nothing
End Sub



'' SIG '' Begin signature block
'' SIG '' MIIVGQYJKoZIhvcNAQcCoIIVCjCCFQYCAQExCzAJBgUr
'' SIG '' DgMCGgUAMGcGCisGAQQBgjcCAQSgWTBXMDIGCisGAQQB
'' SIG '' gjcCAR4wJAIBAQQQTvApFpkntU2P5azhDxfrqwIBAAIB
'' SIG '' AAIBAAIBAAIBADAhMAkGBSsOAwIaBQAEFPNn1ez3RAU5
'' SIG '' ExsMB5pV1AKRBcQDoIIP0TCCBJkwggOBoAMCAQICDxaI
'' SIG '' 8DklXmOOaRQ5B+YzCzANBgkqhkiG9w0BAQUFADCBlTEL
'' SIG '' MAkGA1UEBhMCVVMxCzAJBgNVBAgTAlVUMRcwFQYDVQQH
'' SIG '' Ew5TYWx0IExha2UgQ2l0eTEeMBwGA1UEChMVVGhlIFVT
'' SIG '' RVJUUlVTVCBOZXR3b3JrMSEwHwYDVQQLExhodHRwOi8v
'' SIG '' d3d3LnVzZXJ0cnVzdC5jb20xHTAbBgNVBAMTFFVUTi1V
'' SIG '' U0VSRmlyc3QtT2JqZWN0MB4XDTE1MTIzMTAwMDAwMFoX
'' SIG '' DTE5MDcwOTE4NDAzNlowgYQxCzAJBgNVBAYTAkdCMRsw
'' SIG '' GQYDVQQIExJHcmVhdGVyIE1hbmNoZXN0ZXIxEDAOBgNV
'' SIG '' BAcTB1NhbGZvcmQxGjAYBgNVBAoTEUNPTU9ETyBDQSBM
'' SIG '' aW1pdGVkMSowKAYDVQQDEyFDT01PRE8gU0hBLTEgVGlt
'' SIG '' ZSBTdGFtcGluZyBTaWduZXIwggEiMA0GCSqGSIb3DQEB
'' SIG '' AQUAA4IBDwAwggEKAoIBAQDp6T3f1zcIyR44slJTQm0i
'' SIG '' 8bHEBgRrnv2CdFBDfcagux9O+QJxJrHvQ9iDjEj85w+X
'' SIG '' eprrnN6mow47HEQYdY54pRdp/kkYpOK7XE7+jipUelDw
'' SIG '' 1fbMkeeZedfeeZTXljP+DoO+Ir9jFiyj3Sgbrz2r6pfS
'' SIG '' 8b8EEOc9SEX9H2hlwX9ZmWnAIjEMYm6nXGUBIbBjxCIY
'' SIG '' J+7m/NIAPUcuqLiGVl0E3BMXJW4c30QPFc2326VXdkJv
'' SIG '' AGiCmdLjwd7wi5RXTOwIkCIhziIrmAxC5kKTlJiT7/0G
'' SIG '' 2T+8W5tUPCCx7mrWR3rFq4DpMJre8aQ/VU0KCTSKdSnS
'' SIG '' aa2XD1C/+MoJAgMBAAGjgfQwgfEwHwYDVR0jBBgwFoAU
'' SIG '' 2u1kdBScFDyr3ZmpvVsoTYs8ydgwHQYDVR0OBBYEFI5r
'' SIG '' LTNr9DOnk7MTmqXgCvcSNWqIMA4GA1UdDwEB/wQEAwIG
'' SIG '' wDAMBgNVHRMBAf8EAjAAMBYGA1UdJQEB/wQMMAoGCCsG
'' SIG '' AQUFBwMIMEIGA1UdHwQ7MDkwN6A1oDOGMWh0dHA6Ly9j
'' SIG '' cmwudXNlcnRydXN0LmNvbS9VVE4tVVNFUkZpcnN0LU9i
'' SIG '' amVjdC5jcmwwNQYIKwYBBQUHAQEEKTAnMCUGCCsGAQUF
'' SIG '' BzABhhlodHRwOi8vb2NzcC51c2VydHJ1c3QuY29tMA0G
'' SIG '' CSqGSIb3DQEBBQUAA4IBAQC6MyRAQIx821ifs2CYsvXA
'' SIG '' Mf7rH25Q9grg5OaBrSaHot/9s9r0c/MA+ykbiRsVPttr
'' SIG '' UpMrxKw5gdc8Z1eaOTbgKAia4zlPm4kJf3vFYX9ZiTIl
'' SIG '' CmquGj7woieotsO4h/cWBEhBPVzY7J9NIDEE2WWh7c1p
'' SIG '' B1MWPd02AgqI60DlBjALuBZL3O+8VQn/xj4SLnaz3M5C
'' SIG '' 7/l2V+G3CgVAmFiaXXEWk3GMZYHqb/OJ9/tzrbTnv9mO
'' SIG '' b6oLTyXzuOHV3XWYaIH4qsDRgMLExDmJwfbJnmzXdPnZ
'' SIG '' l/hPwpoKzV6P+Bnp4KWfxPCSIeYteSXJIvnD8DqEV606
'' SIG '' FvRjlBAdXdDGMIIFTDCCBDSgAwIBAgIRAJwQT26bml6F
'' SIG '' RlRQC9FdUgQwDQYJKoZIhvcNAQELBQAwfTELMAkGA1UE
'' SIG '' BhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3Rl
'' SIG '' cjEQMA4GA1UEBxMHU2FsZm9yZDEaMBgGA1UEChMRQ09N
'' SIG '' T0RPIENBIExpbWl0ZWQxIzAhBgNVBAMTGkNPTU9ETyBS
'' SIG '' U0EgQ29kZSBTaWduaW5nIENBMB4XDTE4MDQxNjAwMDAw
'' SIG '' MFoXDTE5MDQxNjIzNTk1OVowgZUxCzAJBgNVBAYTAlVT
'' SIG '' MQ4wDAYDVQQRDAUyMDcyMzELMAkGA1UECAwCTUQxDzAN
'' SIG '' BgNVBAcMBkxhdXJlbDEeMBwGA1UECQwVODQwNSBDaGVy
'' SIG '' cnkgTGF1cmVsIEN0MRswGQYDVQQKDBJEb2N0b3IgQmFj
'' SIG '' a3VwLCBMTEMxGzAZBgNVBAMMEkRvY3RvciBCYWNrdXAs
'' SIG '' IExMQzCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoC
'' SIG '' ggEBANlJE39CJ44QVNO03T1SXOsDtcPNJ2nMV7fhU9PJ
'' SIG '' ScR/2yep4x6oIUxrEu8jdtaotU/+hbt3HGUe7foguQ1v
'' SIG '' UhPl59kxkefl1FEeKKfx49cIh4HcQOwi8IZtvwrbNXhz
'' SIG '' +yOn7zoBc50TMOPc5wlmzOLIANQzlAVAOEhA7+Z/Xe8g
'' SIG '' D1+qa6NLN/2YPJifhEinDj5C5ckJFbIxMjYJ8cvkUtbU
'' SIG '' 5w+stP4KRQNCrfiK37qfbKygqMF0ssdLNGipkSLvZbH7
'' SIG '' P8k51S+qHKcpF9j7cMQUuRnLpzv5VR4YNYUjte5ct1tU
'' SIG '' QHMXbvvBM5RaQ8t+I0RFbjpo2SCH4EjP1Dq1H3MCAwEA
'' SIG '' AaOCAawwggGoMB8GA1UdIwQYMBaAFCmRYP+KTfrr+aZq
'' SIG '' uM/55ku9Sc4SMB0GA1UdDgQWBBS8OfmptvN/YAEKSPai
'' SIG '' fe3HqHLOVjAOBgNVHQ8BAf8EBAMCB4AwDAYDVR0TAQH/
'' SIG '' BAIwADATBgNVHSUEDDAKBggrBgEFBQcDAzARBglghkgB
'' SIG '' hvhCAQEEBAMCBBAwRgYDVR0gBD8wPTA7BgwrBgEEAbIx
'' SIG '' AQIBAwIwKzApBggrBgEFBQcCARYdaHR0cHM6Ly9zZWN1
'' SIG '' cmUuY29tb2RvLm5ldC9DUFMwQwYDVR0fBDwwOjA4oDag
'' SIG '' NIYyaHR0cDovL2NybC5jb21vZG9jYS5jb20vQ09NT0RP
'' SIG '' UlNBQ29kZVNpZ25pbmdDQS5jcmwwdAYIKwYBBQUHAQEE
'' SIG '' aDBmMD4GCCsGAQUFBzAChjJodHRwOi8vY3J0LmNvbW9k
'' SIG '' b2NhLmNvbS9DT01PRE9SU0FDb2RlU2lnbmluZ0NBLmNy
'' SIG '' dDAkBggrBgEFBQcwAYYYaHR0cDovL29jc3AuY29tb2Rv
'' SIG '' Y2EuY29tMB0GA1UdEQQWMBSBEm1yb21tQGRyYmFja3Vw
'' SIG '' Lm5ldDANBgkqhkiG9w0BAQsFAAOCAQEAkiHmwb49B0U+
'' SIG '' 1A6y9JX9zfM5sH4kXoa3YMqrFLOCvOZaAxcMe4hwApy+
'' SIG '' IXJYGfBMxn7+eakvgyQO9TaDFWPpYEvI+jAUaw8aAeUE
'' SIG '' ybpSKFl8Ag6VFg3LbzbuEI3k+RxaWDv/6qJJ1EeIDnN9
'' SIG '' vZCsw58cIei/TSyAuQ2BrRCO7KcjZUTbZi6fWFxbmoVR
'' SIG '' FoUP5ILf1qQJWLCAKXZ0ZZ5ECELCpfM1m2bZh3QS9D+l
'' SIG '' epkT59Kqp6HxtQi0ekfSjNkkySzck8xWE3LBmVk9IDSF
'' SIG '' XFxXtoDczKuRZayhUGeMcCZiyvldKFkCnzWwfEhTpCfT
'' SIG '' zWxX84RIs+6ktaofQFKDADCCBeAwggPIoAMCAQICEC58
'' SIG '' h8wOk0pS/pT9HLfNNK8wDQYJKoZIhvcNAQEMBQAwgYUx
'' SIG '' CzAJBgNVBAYTAkdCMRswGQYDVQQIExJHcmVhdGVyIE1h
'' SIG '' bmNoZXN0ZXIxEDAOBgNVBAcTB1NhbGZvcmQxGjAYBgNV
'' SIG '' BAoTEUNPTU9ETyBDQSBMaW1pdGVkMSswKQYDVQQDEyJD
'' SIG '' T01PRE8gUlNBIENlcnRpZmljYXRpb24gQXV0aG9yaXR5
'' SIG '' MB4XDTEzMDUwOTAwMDAwMFoXDTI4MDUwODIzNTk1OVow
'' SIG '' fTELMAkGA1UEBhMCR0IxGzAZBgNVBAgTEkdyZWF0ZXIg
'' SIG '' TWFuY2hlc3RlcjEQMA4GA1UEBxMHU2FsZm9yZDEaMBgG
'' SIG '' A1UEChMRQ09NT0RPIENBIExpbWl0ZWQxIzAhBgNVBAMT
'' SIG '' GkNPTU9ETyBSU0EgQ29kZSBTaWduaW5nIENBMIIBIjAN
'' SIG '' BgkqhkiG9w0BAQEFAAOCAQ8AMIIBCgKCAQEAppiQY3eR
'' SIG '' NH+K0d3pZzER68we/TEds7liVz+TvFvjnx4kMhEna7xR
'' SIG '' kafPnp4ls1+BqBgPHR4gMA77YXuGCbPj/aJonRwsnb9y
'' SIG '' 4+R1oOU1I47Jiu4aDGTH2EKhe7VSA0s6sI4jS0tj4CKU
'' SIG '' N3vVeZAKFBhRLOb+wRLwHD9hYQqMotz2wzCqzSgYdUjB
'' SIG '' eVoIzbuMVYz31HaQOjNGUHOYXPSFSmsPgN1e1r39qS/A
'' SIG '' JfX5eNeNXxDCRFU8kDwxRstwrgepCuOvwQFvkBoj4l84
'' SIG '' 28YIXUezg0HwLgA3FLkSqnmSUs2HD3vYYimkfjC9G7WM
'' SIG '' crRI8uPoIfleTGJ5iwIGn3/VCwIDAQABo4IBUTCCAU0w
'' SIG '' HwYDVR0jBBgwFoAUu69+Aj36pvE8hI6t7jiY7NkyMtQw
'' SIG '' HQYDVR0OBBYEFCmRYP+KTfrr+aZquM/55ku9Sc4SMA4G
'' SIG '' A1UdDwEB/wQEAwIBhjASBgNVHRMBAf8ECDAGAQH/AgEA
'' SIG '' MBMGA1UdJQQMMAoGCCsGAQUFBwMDMBEGA1UdIAQKMAgw
'' SIG '' BgYEVR0gADBMBgNVHR8ERTBDMEGgP6A9hjtodHRwOi8v
'' SIG '' Y3JsLmNvbW9kb2NhLmNvbS9DT01PRE9SU0FDZXJ0aWZp
'' SIG '' Y2F0aW9uQXV0aG9yaXR5LmNybDBxBggrBgEFBQcBAQRl
'' SIG '' MGMwOwYIKwYBBQUHMAKGL2h0dHA6Ly9jcnQuY29tb2Rv
'' SIG '' Y2EuY29tL0NPTU9ET1JTQUFkZFRydXN0Q0EuY3J0MCQG
'' SIG '' CCsGAQUFBzABhhhodHRwOi8vb2NzcC5jb21vZG9jYS5j
'' SIG '' b20wDQYJKoZIhvcNAQEMBQADggIBAAI/AjnD7vjKO4ne
'' SIG '' DG1NsfFOkk+vwjgsBMzFYxGrCWOvq6LXAj/MbxnDPdYa
'' SIG '' CJT/JdipiKcrEBrgm7EHIhpRHDrU4ekJv+YkdK8eexYx
'' SIG '' biPvVFEtUgLidQgFTPG3UeFRAMaH9mzuEER2V2rx31hr
'' SIG '' IapJ1Hw3Tr3/tnVUQBg2V2cRzU8C5P7z2vx1F9vst/dl
'' SIG '' CSNJH0NXg+p+IHdhyE3yu2VNqPeFRQevemknZZApQIvf
'' SIG '' ezpROYyoH3B5rW1CIKLPDGwDjEzNcweU51qOOgS6oqF8
'' SIG '' H8tjOhWn1BUbp1JHMqn0v2RH0aofU04yMHPCb7d4gp1c
'' SIG '' /0a7ayIdiAv4G6o0pvyM9d1/ZYyMMVcx0DbsR6HPy4uo
'' SIG '' 7xwYWMUGd8pLm1GvTAhKeo/io1Lijo7MJuSy2OU4wqjt
'' SIG '' xoGcNWupWGFKCpe0S0K2VZ2+medwbVn4bSoMfxlgXwya
'' SIG '' iGwwrFIJkBYb/yud29AgyonqKH4yjhnfe0gzHtdl+K7J
'' SIG '' +IMUk3Z9ZNCOzr41ff9yMU2fnr0ebC+ojwwGUPuMJ7N2
'' SIG '' yfTm18M04oyHIYZh/r9VdOEhdwMKaGy75Mmp5s9ZJet8
'' SIG '' 7EUOeWZo6CLNuO+YhU2WETwJitB/vCgoE/tqylSNklzN
'' SIG '' wmWYBp7OSFvUtTeTRkF8B93P+kPvumdh/31J4LswfVyA
'' SIG '' 4+YWOUunMYIEtDCCBLACAQEwgZIwfTELMAkGA1UEBhMC
'' SIG '' R0IxGzAZBgNVBAgTEkdyZWF0ZXIgTWFuY2hlc3RlcjEQ
'' SIG '' MA4GA1UEBxMHU2FsZm9yZDEaMBgGA1UEChMRQ09NT0RP
'' SIG '' IENBIExpbWl0ZWQxIzAhBgNVBAMTGkNPTU9ETyBSU0Eg
'' SIG '' Q29kZSBTaWduaW5nIENBAhEAnBBPbpuaXoVGVFAL0V1S
'' SIG '' BDAJBgUrDgMCGgUAoIGwMBkGCSqGSIb3DQEJAzEMBgor
'' SIG '' BgEEAYI3AgEEMBwGCisGAQQBgjcCAQsxDjAMBgorBgEE
'' SIG '' AYI3AgEVMCMGCSqGSIb3DQEJBDEWBBQ3qRc3cEtOC96F
'' SIG '' uXuLfW6s4uYfYzBQBgorBgEEAYI3AgEMMUIwQKAigCAA
'' SIG '' UwBhAHYAZQBTAGUAdAB0AGkAbgBnAHMAXwBTAFEATKEa
'' SIG '' gBhodHRwOi8vd3d3LmRyYmFja3VwLm5ldCAwDQYJKoZI
'' SIG '' hvcNAQEBBQAEggEACVmYZng4Lv4FNuSdmOhpNPi4XXHe
'' SIG '' K7gzFkkWzVgzQ/w5YOsWkAVVKM1KecpV8sl3YHqyVYKA
'' SIG '' ZKUDUJfByW/CGZTWgIrassgKfDCPph4hamgx02JC6ipD
'' SIG '' tKtdFFads3oYNXVrmaThY6w/JSL77Qld6kB7BiMUW3WN
'' SIG '' 72F8Znyn5+HDFFuclWNkGbLQ+Gic8AYoi0UFHqaBfunj
'' SIG '' fzhWreYOP4EcFr+t91F6sWkLwjtER7wOOyFg8e3Jpf0t
'' SIG '' /Mg99keeLfxO5SjpdKCytwXy61jmGVpco/kAhsv9nwkU
'' SIG '' abZDPewr5QMz7I27NPJxwe5IHT0xGV2yxOWKYdzuRMB/
'' SIG '' erro+6GCAkMwggI/BgkqhkiG9w0BCQYxggIwMIICLAIB
'' SIG '' ATCBqTCBlTELMAkGA1UEBhMCVVMxCzAJBgNVBAgTAlVU
'' SIG '' MRcwFQYDVQQHEw5TYWx0IExha2UgQ2l0eTEeMBwGA1UE
'' SIG '' ChMVVGhlIFVTRVJUUlVTVCBOZXR3b3JrMSEwHwYDVQQL
'' SIG '' ExhodHRwOi8vd3d3LnVzZXJ0cnVzdC5jb20xHTAbBgNV
'' SIG '' BAMTFFVUTi1VU0VSRmlyc3QtT2JqZWN0Ag8WiPA5JV5j
'' SIG '' jmkUOQfmMwswCQYFKw4DAhoFAKBdMBgGCSqGSIb3DQEJ
'' SIG '' AzELBgkqhkiG9w0BBwEwHAYJKoZIhvcNAQkFMQ8XDTE4
'' SIG '' MTIyMzIwNTExOVowIwYJKoZIhvcNAQkEMRYEFJaAVgMX
'' SIG '' iVqwZ1tiz7b03Brl5PirMA0GCSqGSIb3DQEBAQUABIIB
'' SIG '' AH1CiPd2KYfQ/8zL/ba5c5QUD29X0WbPLC1hmG4Di5ut
'' SIG '' EcRSDW182StmfxR1FjLDZE5cc9bOy+Y9gMhx3RNtWWt3
'' SIG '' t2SZgXoVnTI+5iNzvBmDvo6aOoOyRcHMa1nih0ty+EBN
'' SIG '' jlHjjfPl2PpqaK3hmGWNxZ8aLWlVwiPwpW1la2R7Rbkg
'' SIG '' uEZYLoF8hrUf4+L8aC2IZCy2QVzkHJNFUTEEA2eSyS3q
'' SIG '' NMe1E/bqsQEetCX5vwdqqnvNkAYv4P4LpsbOWTShakwE
'' SIG '' lcY2hZe9mxTiX+MeJBFvNsbx5RahQWqmStfCfebcAX6w
'' SIG '' 7ERqPeX4rBSSrimPFmudAW7GCTOOZxieoNI=
'' SIG '' End signature block
